using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

static class Program
{ [STAThread] static void Main() { Game1 game = new Game1(); game.Run(); }

public class Game1 : Game
{ private GraphicsDeviceManager g;
  private ContentManager content;
  private Model model;
  private Matrix world, view, proj;
  private float positionX = 0.0f, positionY = 0.0f, positionZ = 0.0f;
  private float scaleX    = 1.0f, scaleY    = 1.0f, scaleZ    = 1.0f;
  private float rotationX = 0.5f, rotationY = 0.5f, rotationZ = 0.0f;

  public Game1()
  { g = new GraphicsDeviceManager( this );
    content  = new ContentManager( Services );
  }
  protected override void Initialize()
  { g.PreferredBackBufferWidth  = 400;
    g.PreferredBackBufferHeight = 400;
    g.ApplyChanges();
    Window.Title = "Dice";
    g.IsFullScreen = false;
    Window.AllowUserResizing = true;
    base.Initialize();
  }
  protected override void LoadContent()
  { model = content.Load< Model >( "Content\\dice"  );
  }
  protected override void UnloadContent()
  { content.Unload();
  }
  protected override void Update( GameTime gameTime )
  { GamePadState s = GamePad.GetState( PlayerIndex.One );
    if ( !s.IsConnected ) Exit();
    Vector2 ts = s.ThumbSticks.Left;
    rotationY += 0.05f * ts.X;
    rotationX -= 0.05f * ts.Y;
    ts = s.ThumbSticks.Right;
    rotationY += 0.05f * ts.X;
    positionZ -= 0.1f * ts.Y;
    if ( s.DPad.Left  == ButtonState.Pressed ) positionX -= 0.05f;
    if ( s.DPad.Right == ButtonState.Pressed ) positionX += 0.05f;
    if ( s.DPad.Up    == ButtonState.Pressed ) positionY += 0.05f;
    if ( s.DPad.Down  == ButtonState.Pressed ) positionY -= 0.05f;
    if ( s.Buttons.Y  == ButtonState.Pressed ) scaleY *= 1.01f;
    if ( s.Buttons.A  == ButtonState.Pressed ) scaleY *= 0.99f;
    if ( s.Buttons.X  == ButtonState.Pressed ) scaleX *= 0.99f;
    if ( s.Buttons.B  == ButtonState.Pressed ) scaleX *= 1.01f;
    if ( s.Buttons.Back  == ButtonState.Pressed )
    { positionX = positionY = positionZ = 0.0f;
      scaleX    = scaleY    = scaleZ    = 1.0f;
      rotationX = rotationY = rotationZ = 0.0f;
    }
    world = Matrix.CreateScale( scaleX, scaleY, scaleZ ) *
            Matrix.CreateRotationX( rotationX ) *
            Matrix.CreateRotationY( rotationY ) *
            Matrix.CreateRotationZ( rotationZ ) *
            Matrix.CreateTranslation( positionX, positionY, positionZ );
    view =  Matrix.CreateLookAt( new Vector3(0f, 0f, 4f), Vector3.Zero, Vector3.Up );
    proj =  Matrix.CreatePerspectiveFieldOfView( MathHelper.Pi/4, 1f, 0.1f, 200f );
    g.GraphicsDevice.RenderState.CullMode = CullMode.None;
    base.Update( gameTime );
  }

  protected override void Draw( GameTime gameTime )
  { g.GraphicsDevice.Clear( Color.DarkBlue );
    foreach ( BasicEffect effect in model.Meshes[0].Effects )
    { effect.EnableDefaultLighting();
      effect.World      = world;
      effect.View       = view;
      effect.Projection = proj;
      model.Meshes[0].Draw();
    }
  }
} // end of class Game1
} // end of class Program